/**
* BinaryNode.h - A node with two neighbours
* 
**/
#ifndef BinaryNode_h
#define BinaryNode_h

template <class O>
class BinaryNode{

  public:

    /**
    * Constructor: Create a new BinaryNode with a given element
    * @param data - The element to be contained within the BinaryNode
    * @post this->data == data && this->left == NULL && this->right == NULL
    **/
    BinaryNode( const O& data );

    /**
    * Constructor: Create a new BinaryNode with a given element
    * @param data - The element to be contained within the BinaryNode
    * @param left - Left neighbour of this BinaryNode
    * @param right - Right neighbour of this BinaryNode
    * @post this->data == data && this->left == left && this->right == right
    **/
    BinaryNode( const O& data, BinaryNode<O>* left, BinaryNode<O>* right );

    
    O data;                 /* Data element of the BinaryNode */  
    BinaryNode<O>* left;    /* Left neighbour */
    BinaryNode<O>* right;   /* Right neightbour */
};

template <class O>
BinaryNode<O>::BinaryNode(const O& data) : data(data), left(NULL), right(NULL){
}

template <class O>
BinaryNode<O>::BinaryNode(const O& data, BinaryNode<O>* left, 
                          BinaryNode<O>* right) 
                          : data(data), left(left), right(right){
}

#endif
