/**
* List.h - A List interface class
* 
**/
#ifndef List_h
#define List_h

template <class O>
class List{

  public:    

    /**
    * Insert a new element into the List at a given index.
    * @param i - The index at wich to put the new element
    * @param o - The new element.
    * @pre 0 <= i 
    */
    virtual void insert( const unsigned int& i, const O& o ) = 0;
    
    /**
    * Remove an element at a given index from the list.
    * @param i - The index of the element to be removed
    * @pre 0 <= i && i < length()
    */
    virtual void remove( const unsigned int& i ) = 0;

    /**
    * Retrieve an element at the given index from the List
    * @param i - The index of the element to be retreived.
    * @pre 0 <= i && i < length()
    */
    virtual O get( const unsigned int& i ) const = 0; 
    
    /**
    * Returns the number of elements in the List.
    * @return The number of elements in the List
    */
    virtual unsigned int length() const = 0;
    
    /** Destructor */
    virtual ~List(){}

};

#endif
